/*********************************************************
 * Copyright (C) 2021 VMware, Inc. All rights reserved.
 *********************************************************/

/*
 * vmwareHzEventHandler.h
 *
 *      Event related definitions such as event types, handlers.
 */

#ifndef VMW_HORIZON_EVENT_HANDLER_H
#define VMW_HORIZON_EVENT_HANDLER_H

#include <cstddef>
#include "vmwareHzTypes.h"


namespace vmware {
namespace horizon {
namespace client {
namespace protocolconnection {


/*
 * The events that are used in the remote session.
 * Each event has a corresponding event handler.
 * If the callers want to process the event, then they need to provide the handler,
 * otherwise the event is omitted.
 */
enum class HzEventType {
   UNKNOWN = -1,
   ConnectionDisconnected = 0,
   ConnectionPending = 1,
   ConnectionConnected = 2,
   RelativeMouseStatusChanged = 3,
   UsbDesktopPidReceived = 4,
   SupportSyncKeyboardLED = 5,
};


// The prototype of the handler to process the event in HzEventType.
using HzEventHandler = int(*)(HzEventType,
                              void *,
                              size_t,
                              void *);

/*
 * Arguments used to process the following connection state events:
 * - ConnectionConnected
 * - ConnectionPending
 * - ConnectionDisconnected
 */
typedef struct {
   size_t size;                              // The size of the struct.
   HzSessionConnectionState state;           // The connection state.
   HzSessionConnectionStateReason reason;    // The reason changed to the state.
} HzConnectionStateChangedEventArgs;

// Arguments used to process the RelativeMouseStatusChanged event.
typedef struct {
   size_t size;   // The size of the struct.
   bool enabled;  // Indicate whether relative mouse is enabled or not.
} HzEnableRelativeMouseEventArgs;

// Arguments used to process the UsbDesktopPidReceived event.
typedef struct {
   size_t size;             // The size of the struct.
   char *desktopPid;        // Indicate desktopPid.
} HzUsbDesktopPidReceivedEventArgs;

// Arguments used to process the SupportSyncToggleKeys event.
typedef struct {
   size_t size;     // The size of the struct.
   bool supported;  // Indicate whether sync keyboard led is supported or not.
} HzSupportSyncKeyboardLEDEventArgs;

} // namespace protocolconnection
} // namespace client
} // namespace horizon
} // namespace vmware

#endif // VMW_HORIZON_EVENT_HANDLER_H
