/*********************************************************
 * Copyright (C) 2019-2021 VMware, Inc. All rights reserved.
 *********************************************************/

/*
 * vmwareHzProtocolConnection.h
 *
 *    Contains public APIs to set up and manage the protocol connection.
 */


#ifndef VMW_HORIZON_SDK_PROTOCOL_CONNECTION_H
#define VMW_HORIZON_SDK_PROTOCOL_CONNECTION_H


#include <memory>
#include "vmwareHzTypes.h"
#include "vmwareHzEventHandler.h"


namespace vmware {
namespace horizon {
namespace client {

namespace internal {
class ProtocolConnection;
}

class HzProtocolConnection {
public:
   HzProtocolConnection() noexcept;

   // Set the id of the launch item to be connected.
   void SetLaunchItemId(const char *launchItemId);

   // Set the broker address of the current broker connection.
   void SetBrokerAddress(const char *brokerAddress);

   // Set the port of the current broker connection.
   void SetBrokerPort(unsigned short brokerPort);

   // Set the agent address for the current protocol connection.
   void SetAgentAddress(const char *agentAddress);

   // Set the agent FQDN for the current protocol connection.
   void SetAgentFQDN(const char *agentFQDN);

   // Set the agent port for the current protocol connection.
   void SetAgentPort(unsigned short agentPort);

   // Set the token to connect to agent for the current protocol connection.
   void SetAgentToken(const char *agentToken);

   // Set whether the current protocol connection is for remote application or not.
   void SetIsAppSession(bool isAppSession);

   // Set the window handle for remote session window.
   void SetRemoteWindowHandle(HzWindowHandle handle);

   // Set handler and data for the specified event.
   void SetEventHandler(protocolconnection::HzEventType eventType,
                        protocolconnection::HzEventHandler eventHandler,
                        void *eventData);

   // Connect to agent.
   bool Connect();

   // Disconnect from the current agent connection.
   void Disconnect();

   // Set client side display topology to the RMKS layer.
   void SendDisplayTopology(const int *selectedMonitorIndexList,
                            size_t selectedMonitorIndexNum,
                            int x,
                            int y,
                            int width,
                            int height);

   // Set the monitor indexes.
   void SetMonitorIndexes(const int *indexList,
                          size_t indexNum);

   // Set the monitor rectangle.
   void SetMonitorRect(const HzWindowRectPtr rect);

   // Set whether H264 for Blast is enabled or not.
   void SetBlastH264Enabled(bool enabled);

   // Set whether H264 YUV444 for Blast is enabled or not.
   void SetBlastH264YUV444Enabled(bool enabled);

   // Set whether HEVC for Blast is enabled or not.
   void SetBlastHEVCEnabled(bool enabled);

   // Send Ctrl+Alt+Del to the remote session.
   void SendCtrlAltDel();

   // Set whether the relative mouse is enabled or not.
   void EnableRelativeMouse(bool enabled);

   // Request to grab the input in the remote session.
   void RequestGrab();

   // Request to ungrab the input from the remote session.
   void RequestUngrab(bool hardUngrab);

   // Set whether DPI scaling is enabled or not in the remote window.
   void SetDpiScalingEnabled(bool enabled);

   // Set whether sending Ctrl+Alt+Del to the agent is allowed or not.
   void SetSendCadToVMAllowed(bool allowed);

   // Set the hooked keys used in the remote window.
   void SetHookedKeys(const HzHotKey *hookedKeyList,
                      size_t hookedKeyNum);

   // Set the security level when doing SSL handshake between client and agent.
   void SetSecurityLevel(HzSecurityLevel securityLevel);

   /*
    * Set whether it is allowed to use proxy for the SSL handshake when the
    * client side setting up protocol connection with agent.
    */
   void SetSslProxyAllowed(bool allowed);

   // Set whether TLS 1.1 is enabled for the protocol connection.
   void SetTlsv11Enabled(bool enabled);

   // Set whether TLS 1.2 is enabled for the protocol connection.
   void SetTlsv12Enabled(bool enabled);

   // Set the SSL cipher control string for the protocol connection.
   void SetSslCipherControlString(const char *string);

   // Set the protocol to be used when connecting to agent.
   void SetProtocol(HzSessionProtocol protocol);

   // Set the VDP service listener for the protocol connection.
   void SetVdpserviceListener(const char *vdpserviceListener);

   // Set the proxy to be used when connecting to agent.
   void SetAgentProxy(const char *proxy);

   // Set whether Multimedia Redirection is enabled (disabled by default).
   void SetMMREnabled(bool enabled);

   // Set whether Sync Keyboard LED status is enabled (disabled by default).
   void SetSyncKeyboardLEDStatus(bool enabled);

private:
   std::shared_ptr<internal::ProtocolConnection> mImpl;
};


} // namespace client
} // namespace horizon
} // namespace vmware


#endif // VMW_HORIZON_SDK_PROTOCOL_CONNECTION_H
